import logging

from portal.database.models import Course, User, ProjectState
from portal.service import general
from portal.service.groups import copy_group
from portal.service.projects import copy_project
from portal.service.roles import copy_role
from portal.tools import time

log = logging.getLogger(__name__)


def copy_course(source: Course, target: Course, config: dict) -> Course:
    """Copies course and it's other resources (roles, groups, projects)
    The other resources that should be copied are specified in the config

    Args:
        source(Course): From which course the resources will be copied
        target(Course): To which course the resources will be copied
        config(dict): Configuration to specify which resource should be copied

    Returns(Course): Copied course instance (target)
    """
    if config.get('roles'):
        for role in source.roles:
            copy_role(role, target, with_users=config['roles'])
    if config.get('groups'):
        for group in source.groups:
            copy_group(group, target, with_users=config['groups'])
    if config.get('projects'):
        for project in source.projects:
            copy_project(project, target)

    general.write_entity(target)
    log.info(f"[IMPORT] From {source.id} to:  {target.id}.")

    return target


def delete_course(course: Course):
    """Deletes course

    Args:
        course(Course): Course instance
    """
    log.info(f"[DELETE] Course: {course.id} ({course.codename})")
    general.delete_entity(course)


def __set_course_props(course: Course, data: dict):
    return general.update_entity(course, data, allowed=['name', 'codename'])


def update_course(course: Course, data: dict) -> Course:
    """Updates course

    Args:
        course(Course): Course instance
        data(dict): Dictionary containing data which should be changed

    Returns(Course): Updated course instance
    """
    __set_course_props(course=course, data=data)
    log.info(f"[UPDATE] Course {course.id} ({course.codename}): {course}.")
    return course


def create_course(data: dict) -> Course:
    """Creates new course

    Args:
        data(dict): Course representation in dictionary

    Returns(Course): Course instance
    """
    course = Course()
    __set_course_props(course=course, data=data)
    log.debug(f"[CREATE] Course {course.id}: {course}")
    return course


def update_notes_token(course: Course, token: str) -> Course:
    """Updates notes access token of a course.

    Args:
        course(Course): Course instance for which the token is set
        token(str): The new token

    Returns(Course): Course instance
    """
    course.notes_access_token = token
    general.write_entity(course)
    log.info(f"[UPDATE] Notes access token ({course.codename}) [{course.id}]: {token}")
    return course


def filter_projects_from_course(projects: list, course: Course,
                                state: ProjectState = ProjectState.ACTIVE) -> list:
    """Filters projects for the course based on their state.

    Args:
        state(ProjectState): State the project must be in to be included in the result. Defaults to ACTIVE.
        projects(list): Project list
        course(Course): Course instance

    Returns(list): Filtered list of  projects
    """

    def is_active(p):
        return p.state(time.NOW()) == state

    active_projects_id = [p.id for p in course.projects if is_active(p)]
    projects = [p for p in projects if p['id'] in active_projects_id]
    return projects


def filter_roles_from_course(roles: list, course: Course, user: User) -> list:
    """Gets roles in a course the user has

    Args:
        roles(list): List of roles to filter
        course(Course): Course instance
        user(User): User instance

    Returns(list): Filtered list of roles
    """
    user_roles = user.get_roles_in_course(course=course)
    user_roles_id = [r.id for r in user_roles]
    return [r for r in roles if r['id'] in user_roles_id]


def filter_groups_from_course(groups: list, course: Course, user: User) -> list:
    """Filters groups of a course. Returns only groups the user is part of.

    Args:
        groups(list): List of groups to filter
        course(Course): Course instance
        user(User): User instance

    Returns(list): Filtered list of groups
    """
    user_groups = user.get_groups_in_course(course=course)
    user_groups_id = [g.id for g in user_groups]
    return [g for g in groups if g['id'] in user_groups_id]


def filter_course_dump(course: Course, dump: dict, user: User) -> dict:
    log.debug(f"[FILTER] Course: {dump}")
    filtered_dump = {
        'projects': filter_projects_from_course(dump['projects'], course),
        'roles': filter_roles_from_course(dump['roles'], course, user),
        'groups': filter_groups_from_course(dump['groups'], course, user)
    }
    return filtered_dump


def find_all_courses():
    return Course.query.all()
